<?xml version="1.0"?>
<!-- Output layout inspired by Morten Brix Pedersen's "pisg" -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
   <xsl:template match="/">
      <html>
         <head>
            <title><xsl:value-of select="stats/title" /></title>
            <link rel="Stylesheet" type="text/css" href="../ProgMon.css" />
         </head>
         <body>
            <div class="ProgMon">
               <xsl:apply-templates />
            </div>
            <div class="signature">
               <hr />
               Statistics generated by:<br /><br />
               <table>
                  <tr>
                     <td class="p1 sigcell">P</td>
                     <td class="r2 sigcell">r</td>
                     <td class="o3 sigcell">o</td>
                     <td class="g4 sigcell">g</td>
                     <td class="m5 sigcell">M</td>
                     <td class="o6 sigcell">o</td>
                     <td class="n7 sigcell">n</td>
                  </tr>
                  <tr>
                     <td colspan="7"><a name="sig">ProgMon</a></td>
                  </tr>
               </table>
            </div>
         </body>
      </html>
   </xsl:template>
   <xsl:template match="title">
      <div class="pageheader">
         <div class="title"><xsl:value-of select="." /></div><br />
      </div>
   </xsl:template>
   <xsl:template match="recorded">
      <div class="pageheader">
         Statistics generated on <xsl:value-of select="." /><br />
         <hr />
      </div>
   </xsl:template>
   <xsl:template match="graph">
      <xsl:variable name="floatWidthDefault"
                    select="(string-length(@max) +
                             string-length(@unit) + 15) * 8" />
      <xsl:variable name="floatWidthCalc" select="(5 + count(datum)) * 7" />
      <xsl:variable name="floatWidth">
         <xsl:choose>
            <xsl:when test="$floatWidthCalc &lt; $floatWidthDefault">
               <xsl:value-of select="$floatWidthDefault" />
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="$floatWidthCalc" />
            </xsl:otherwise>
         </xsl:choose>
      </xsl:variable>
      <div class="floater" style="width: {$floatWidth}px;">
         <table>
            <tr>
               <td class="heading" colspan="1">
                  <xsl:value-of select="@label" />
               </td>
            </tr>
            <tr>
               <td class="graphcell">
                  <table>
                     <tr>
                        <xsl:apply-templates match="datum" />
                     </tr>
                  </table>
               </td>
            </tr>
            <tr>
               <td nowrap="nowrap">
                  Maximum value:
                  <xsl:value-of select="@max" /><xsl:value-of select="@unit" />
               </td>
            </tr>
         </table>
      </div>
   </xsl:template>
   <xsl:template match="datum">
      <td>
         <img width="5" height="{.}" src="../resources/{../@image}" />
      </td>
   </xsl:template>
   <xsl:template match="messages">
      <div class="floater">
         <table>
            <tr>
               <td class="heading" colspan="8">Window Messages Intercepted</td>
            </tr>
            <tr>
               <td class="header">Message Name</td>
               <td class="header">Window Handle</td>
               <td class="header">Total<br />Calls</td>
               <td class="header">Total Time<br />/&#181;s</td>
               <td class="header">Minimum<br />Time /&#181;s</td>
               <td class="header">Mean Time<br />/&#181;s</td>
               <td class="header">Maximum<br />Time /&#181;s</td>
               <td class="header">Total<br />Time %</td>
            </tr>
            <xsl:apply-templates match="message" />
         </table>
      </div>
   </xsl:template>
   <xsl:template match="sample">
      <tr>
         <td><xsl:value-of select="@name" /></td>
         <td><xsl:value-of select="window" /></td>
         <td class="right"><xsl:value-of select="number" /></td>
         <td class="right"><xsl:value-of select="total" /></td>
         <td class="right"><xsl:value-of select="min" /></td>
         <td class="right"><xsl:value-of select="mean" /></td>
         <td class="right"><xsl:value-of select="max" /></td>
         <td class="right"><xsl:value-of select="percent" /></td>
      </tr>
   </xsl:template>
   <xsl:template match="other">
      <div class="floater">
         <table>
            <tr>
               <td class="heading" colspan="2">Other Interesting Numbers</td>
            </tr>
            <xsl:apply-templates match="stat" />
         </table>
      </div>
   </xsl:template>
   <xsl:template match="stat">
      <tr>
         <td><xsl:value-of select="@label" />:</td>
         <td><xsl:value-of select="." /></td>
      </tr>
   </xsl:template>
</xsl:stylesheet>
