#pragma once

#include "Profiler.hh"

//! The message epoch number.
/*! For every MESSAGE_EPOCH messages handled, check to see if there are new
 *  windows. It's not pretty but it works.
 */
#define MESSAGE_EPOCH 100

/*! Because windows.h includes a lot of other files by default, the resulting
 *  object code can be quite large even if most of it isn't used. For this
 *  reason these defines are provided to remove the largest unused portions of
 *  the code at compile time.
 */
#define WIN32_LEAN_AND_MEAN //!< Removes most of windows.h.
#define VC_EXTRALEAN //!< Slims down windows.h a lot.
#define NOGDI //!< Removes GDI support from windows.h.
#define NOSOUND //!< Removes media support from windows.h.

#include "windows.h"
#include "stdint.h"

#include <fstream>
#include <map>

using namespace std;

//! A mnemonic for map<HWND, uint32_t>.
/*! Since there are a lot of similar types to uint32_t defined in stdint.h, it
 *  is best to remove the possibility of inserting a bug in the code while using
 *  iterators, for instance.
 */
typedef map<HWND, uint32_t> window_map;

//! An internal map of windows and old window procedures.
window_map mWindows;

//! Since this is used so much, it is stored as a global variable.
uint32_t uProcess = 0;

//! The number of messages processed in total.
uint32_t uMessages = 0;

//! The first window found in the new process.
HWND hFirst = 0;
//! Store the desktop window.
HWND hDesktop = GetDesktopWindow();

LRESULT CALLBACK HookProc(HWND, UINT, WPARAM, LPARAM);
