#pragma once

/*! Because windows.h includes a lot of other files by default, the resulting
 *  object code can be quite large even if most of it isn't used. For this
 *  reason these defines are provided to remove the largest unused portions of
 *  the code at compile time.
 */
#define WIN32_LEAN_AND_MEAN //!< Removes most of windows.h.
#define VC_EXTRALEAN //!< Slims down windows.h a lot.
#define NOGDI //!< Removes GDI support from windows.h.
#define NOSOUND //!< Removes media support from windows.h.

#include "windows.h" //!< Includes lots of useful Windows APIs.
#include "tlhelp32.h" //!< For get_process_handle functions.
#include "stdint.h" //!< These are not in the C++ standard. Yet.

#include <string>
#include <iostream>

using namespace std;

//! A shorter name for LPTHREAD_START_ROUTINE.
/*! LPTHREAD_START_ROUTINE is really too long a name to make neatly laid out
 *  code possible, even given my usual layout contortion habits, so this typedef
 *  is only here to aid legibility.
 */
typedef LPTHREAD_START_ROUTINE LPSTART;
