#pragma once

#define SCALAR 50 //!< The actual pixel height of the graphs.

#define WIN32_LEAN_AND_MEAN //!< Removes most of windows.h.
#define VC_EXTRALEAN //!< Slims down windows.h a lot.
#define NOGDI //!< Removes GDI support from windows.h.
#define NOSOUND //!< Removes media support from windows.h.

#include "windows.h"

#include "MessageTable.hh"
#include "RingBuffer.hh"

#include <map>

using namespace std;

//! An interface to write stats to a file.
struct StatsWriter {
   virtual const string create_filename(const string) const = 0;
   virtual const string header(const string, const tm *) const = 0;
   virtual const string graph(const string, BlackHoleRingBuffer<double, 100> &,
                              const string, const string) const;
   virtual const string graph(const string, BlackHoleRingBuffer<uint16_t,
                                                                100> &,
                              const string, const string) const;
   virtual const string graph(const string, BlackHoleRingBuffer<uint32_t,
                                                                100> &,
                              const string, const string) const;
   virtual const string sample_data() const = 0;
   virtual const string other_nums() const = 0;
   virtual const string footer() const = 0;
};

//! A concrete class to write stats to a file in XML.
class XmlStatsWriter : public StatsWriter {
   private:
      const string date_suffix(uint8_t) const;
      const string make_stat(const string, const string) const;
   public:
      const string create_filename(const string) const;
      const string header(const string, const tm *) const;
      const string graph(const string, BlackHoleRingBuffer<double, 100> &,
                         const string, const string) const;
      const string graph(const string, BlackHoleRingBuffer<uint16_t, 100> &,
                         const string, const string) const;
      const string graph(const string, BlackHoleRingBuffer<uint32_t, 100> &,
                         const string, const string) const;
      const string sample_data() const;
      const string other_nums() const;
      const string footer() const;
};
