#include "../include/MessageTable.hh"

#include <fstream>
#include <sstream>

map<uint16_t, string> MessageTable::mMessages;

//! A simple default constructor.
/*! When the \c MessageTable is created, load the message data.
 */
MessageTable::MessageTable() {
   load("Messages.txt");
}

//! Get the name corresponding to a message number.
/*! \param[in] uint16_t uMsg The number to find.
 *  \return The corresponding name.
 */
const string MessageTable::get_name(const uint16_t uMsg) {
   stringstream sMsg;
   sMsg << "Message #" << uMsg;
   return mMessages.find(uMsg) != mMessages.end() ? mMessages[uMsg]
                                                  : sMsg.str();
}

//! Get the number corresponding to a message name.
/*! \param[in] string sMsg The name to find.
 *  \return The corresponding number.
 */
const uint16_t MessageTable::get_number(const string sMsg) {
   bool bFound = false;
   map<uint16_t, string>::iterator mIter = mMessages.begin();
   for ( ; mIter != mMessages.end(); mIter++) {
      if ((*mIter).second == sMsg) {
         bFound = true;
         break;
      }
   }
   return bFound ? (*mIter).first : -1;
}

//! Load the message data.
/*! \param[in] string sFile The file name to open.
 */
void MessageTable::load(const string sFile) {
   ifstream iFile;
   iFile.open(sFile.c_str(), ios::in);
   if (iFile.good()) {
      string sLine = "";
      string sTemp = "";
      size_t sEqPos = 0;
      uint8_t uRadix = 0;
      while (!iFile.eof()) {
         getline(iFile, sLine);
         sEqPos = sLine.find('=');
         if (sEqPos != string::npos) {
            uRadix = (sLine.substr(sEqPos + 1).find('$') != string::npos) ? 16
                                                                          : 10;
            sTemp = sLine.substr(sLine.find_last_of(" =$") + 1);
            sTemp = sTemp.substr(0, sTemp.length() - 1);
            mMessages[strtol(sTemp.c_str(), 0,
                             uRadix)] = sLine.substr(0,
                                                     sLine.find_first_of(" ="));
         }
      }
      iFile.close();
   }
}
