#include "../include/StatsWriter.hh"
#include "../include/Profiler.hh"

#include <sstream>
#include <fstream>

// TO DO: doxygen tags in this file. Also Profiler.cc remains.

// XmlStatsWriter class ////////////////////////////////////////////////////////

const string XmlStatsWriter::make_stat(const string sLabel,
                                       const string sStat) const {
   return "      <stat label=\"" + sLabel + "\">" + sStat + "</stat>\n";
}

const string make_sample(const MessageTable &mTable, const Record &rCurrent) {
   char cTemp[10];
   stringstream sConstruct;
   if (rCurrent.is_valid()) {
      pair<HWND, uint32_t> pIdent = rCurrent.get_ident();
      sConstruct << "      <sample name=\"" << mTable.get_name(pIdent.second)
                 << "\">" << endl;
      sConstruct << "         <window>" << pIdent.first << "</window>" << endl;
      sConstruct << "         <number>" << rCurrent.get_data() << "</number>"
                 << endl;
      sprintf(cTemp, "%5.01f", rCurrent.get_data() * 1000000 *
                               rCurrent.get_average() /
                               Profiler::get_ticks_per_second());
      sConstruct << "         <total>" << cTemp << "</total>" << endl;
      sprintf(cTemp, "%5.01f", 1000000 * rCurrent.get_min() /
                               Profiler::get_ticks_per_second());
      sConstruct << "         <min>" << cTemp << "</min>" << endl;
      sprintf(cTemp, "%5.01f", 1000000 * rCurrent.get_average() /
                               Profiler::get_ticks_per_second());
      sConstruct << "         <mean>" << cTemp << "</mean>" << endl;
      sprintf(cTemp, "%5.01f", 1000000 * rCurrent.get_max() /
                               Profiler::get_ticks_per_second());
      sConstruct << "         <max>" << cTemp << "</max>" << endl;
      sprintf(cTemp, "%3.05f", rCurrent.get_percent());
      sConstruct << "         <percent>" << cTemp << "</percent>" << endl;
      sConstruct << "      </sample>" << endl;
   }
   return sConstruct.str();
}

const string XmlStatsWriter::create_filename(const string sTime) const {
   return "results\\" + sTime + ".xml";
}

const string XmlStatsWriter::date_suffix(uint8_t uDate) const {
   string sRetVal = "th";
   if (uDate % 10 == 1 && uDate % 100 != 11) {
      sRetVal = "st";
   }
   if (uDate % 10 == 2 && uDate % 100 != 12) {
      sRetVal = "nd";
   }
   if (uDate % 10 == 3 && uDate % 100 != 13) {
      sRetVal = "rd";
   }
   return sRetVal;
}

const string XmlStatsWriter::header(const string sName, const tm *tTm) const {
   string sDays[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
                      "Friday", "Saturday"};
   string sMonths[12] = {"January", "February", "March", "April", "May", "June",
                         "July", "August", "September", "October", "November",
                         "December"};
   char cTemp[256] = "";
   sprintf(cTemp, "%s %d%s %s %4d at %02d:%02d:%02d UTC",
           sDays[tTm->tm_wday].c_str(), tTm->tm_mday,
           date_suffix(tTm->tm_mday).c_str(), sMonths[tTm->tm_mon].c_str(),
           tTm->tm_year + 1900, tTm->tm_hour, tTm->tm_min, tTm->tm_sec);
   stringstream sConstruct;
   sConstruct << "<?xml version=\"1.0\" encoding=\"ISO8859-1\" ?>" << endl;
   sConstruct << "<!DOCTYPE stats SYSTEM \"ProgMon.dtd\">" << endl;
   sConstruct << "<?xml-stylesheet type=\"text/xsl\" href=\"../ProgMon.xsl\"?>";
   sConstruct << endl << "<stats>" << endl;
   sConstruct << "   <title>Statistics for " << sName << "</title>" << endl;
   sConstruct << "   <recorded>" << cTemp << "</recorded>" << endl;
   return sConstruct.str();
}

const string XmlStatsWriter::footer() const {
   return "</stats>\n";
}

const string XmlStatsWriter::graph(const string sName,
                                   BlackHoleRingBuffer<double, 100> &rBase,
                                   const string sUnit,
                                   const string sImage) const {
   double dMax = 0;
   double dCurrent = 0;
   stringstream sConstruct;
   for (uint8_t uCount = rBase.size(); uCount--; ) {
      rBase >> dCurrent;
      if (dCurrent > dMax) {
         dMax = dCurrent;
      }
      sConstruct << "      <datum>" << (SCALAR * dCurrent / dMax) << "</datum>"
                 << endl;
   }
   sConstruct << "   </graph>" << endl;
   stringstream sHeader;
   sHeader << "   <graph image=\"" << sImage << "\" unit=\"" << sUnit
           << "\" max=\"" << dMax << "\" label=\"" << sName << "\">" << endl;
   return sHeader.str() + sConstruct.str();
}

const string XmlStatsWriter::graph(const string sName,
                                   BlackHoleRingBuffer<uint16_t, 100> &rBase,
                                   const string sUnit,
                                   const string sImage) const {
   uint16_t uMax = 0;
   uint16_t uCurrent = 0;
   stringstream sConstruct;
   for (uint8_t uCount = rBase.size(); uCount--; ) {
      rBase >> uCurrent;
      if (uCurrent > uMax) {
         uMax = uCurrent;
      }
      sConstruct << "      <datum>" << (SCALAR * uCurrent / (double)uMax)
                 << "</datum>" << endl;
   }
   sConstruct << "   </graph>" << endl;
   stringstream sHeader;
   sHeader << "   <graph image=\"" << sImage << "\" unit=\"" << sUnit
              << "\" max=\"" << uMax << "\" label=\"" << sName << "\">" << endl;
   return sHeader.str() + sConstruct.str();
}

const string XmlStatsWriter::graph(const string sName,
                                   BlackHoleRingBuffer<uint32_t, 100> &rBase,
                                   const string sUnit,
                                   const string sImage) const {
   uint32_t uMax = 0;
   uint32_t uCurrent = 0;
   stringstream sConstruct;
   for (uint8_t uCount = rBase.size(); uCount--; ) {
      rBase >> uCurrent;
      if (uCurrent > uMax) {
         uMax = uCurrent;
      }
      sConstruct << "      <datum>" << (SCALAR * uCurrent / (double)uMax)
                 << "</datum>" << endl;
   }
   sConstruct << "   </graph>" << endl;
   stringstream sHeader;
   sHeader << "   <graph image=\"" << sImage << "\" unit=\"" << sUnit
           << "\" max=\"" << uMax << "\" label=\"" << sName << "\">" << endl;
   return sHeader.str() + sConstruct.str();
}

const string XmlStatsWriter::sample_data() const {
   stringstream sConstruct;
   MessageTable mTable;
   sConstruct << "   <messages>" << endl;
   for (uint16_t uCount = 0; uCount < MAX_PROFILER_SAMPLES; uCount++) {
      sConstruct << make_sample(mTable, Sample::rSamples[uCount]);
   }
   sConstruct << "   </messages>" << endl;
   return sConstruct.str();
}

const string XmlStatsWriter::other_nums() const {
   stringstream sConstruct;
   sConstruct << "   <other>" << endl;
   stringstream sTemp;
   char cTime[16];
   sprintf(cTime, "%.2f seconds", Sample::pProfiler->get_total_run_time());
   sConstruct << make_stat("Total monitoring time", cTime);
   sTemp << Sample::get_total_samples();
   sConstruct << make_stat("Total samples taken", sTemp.str());
   sTemp.str("");
   int32_t iObjects = 0;
   iObjects = Sample::pProfiler->get_current_user_objects() -
              Sample::pProfiler->get_stored_user_objects();
   sTemp << iObjects;
   sConstruct << make_stat("User objects leaked", sTemp.str());
   sTemp.str("");
   iObjects = Sample::pProfiler->get_current_gdi_objects() -
              Sample::pProfiler->get_stored_gdi_objects();
   sTemp << iObjects;
   sConstruct << make_stat("GDI objects leaked", sTemp.str());
   sConstruct << "   </other>" << endl;
   return sConstruct.str();
}
